#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.prepare
Called from dlotter.__main__
"""
import sys
import argparse
from dmit import ostools

class prepare:
    """Class for preparing input
    """

    def __init__(self, args:argparse.Namespace) -> None:
        """Constructor for prepare class

        Parameters
        ----------
        args : argparse.Namespace
            Input to dlotter from command line
        """

        if args.cmd == 'plot':
            self.valid_parameters = ['t2m', 'w10m', 'precip', 'slp', 'td2m',
                                     'tcc', 'lmhc', 'snow', 'ws']
            self.check_meta(args)
            self.files_to_read = self.find_files_to_read(args)

        # if args.cmd == 'epsmeteogram':
        #     self.files_to_read = self.find_files_to_read(args, epsmode=True)

        return


    def check_meta(self, args:argparse.Namespace) -> None:
        """Checks if it safe to proceed or user needs to give change inputs

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter.__main__
        """

        dir_state = ostools.does_dir_exist(args.directory)
        if not dir_state:
            print("Input directory: {}, does not exist".format(args.directory), flush=True)
            sys.exit(1)

        allowed_found = False
        parameters = args.parameters.split(':')
        for p in parameters:
            if p in self.valid_parameters:
                allowed_found = True
        if not allowed_found:
            print("No valid parameters found in input argument", flush=True)
            sys.exit(1)

        return


    def find_files_to_read(self, args:argparse.Namespace, epsmode:bool = False) -> list:
        """Finds the file(s) to read and plot from

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter.__main__

        Returns
        -------
        list
            List with path(s) to file(s)
        """

        directory = args.directory

        if not epsmode:
            if args.limit_files > 0:
                inorder = True
            else:
                inorder = False

        if epsmode:
            recursive = True
            inorder = True
        else:
            recursive = False

        files = ostools.find_files(directory,
                                prefix=args.prefix,
                                postfix=args.postfix,
                                recursive=recursive,
                                onlyfiles=True,
                                fullpath=True,
                                olderthan=None,
                                inorder=inorder)

        if not epsmode:
            if args.limit_files > 0:
                if args.limit_files >= len(files):
                    limit = len(files)
                else:
                    limit = args.limit_files

                files = files[0:limit]

        if epsmode:
            epsfiles = []
            for member in range(args.members):
                for f in range(args.files_per_member):
                    #Follows: /something/mbr000/000
                    epsfiles.append("{}/mbr{:03d}/{:03d}".format(directory,member,f))

            if len(epsfiles) != len(files):
                print('Number of automatic found files was different than specified', flush=True)
            files = epsfiles


        return files



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.plot
Called from dlotter.__main__
"""
import sys
import argparse
import xarray as xr
import numpy as np
import datetime as dt

import matplotlib
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap

import cartopy.crs as ccrs
import cartopy.feature as cfeature


class plot:
    """Class for creating geographic plots
    """

    def __init__(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Constructor for plot class

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        self.projections(args)

        parameters = args.parameters
        avail_parameters = list(data.data_vars)

        self.nt = data.dims['time']

        print('- Plotting', flush=True)

        if 't2m' in parameters:
            if 't2m' not in avail_parameters:
                print('t2m was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_t2m(args, data)

        if 'td2m' in parameters:
            if 'td2m' not in avail_parameters:
                print('td2m was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_td2m(args, data)

        if 'w10m' in parameters:
            if 'u10m' not in avail_parameters or 'v10m' not in avail_parameters:
                print('u10m/v10m was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_w10m(args, data)

        if 'precip' in parameters:
            if 'precip' not in avail_parameters:
                print('precip was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_precip(args, data)

        if 'slp' in parameters:
            if 'slp' not in avail_parameters:
                print('slp was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_slp(args, data)

        if 'tcc' in parameters:
            if 'tcc' not in avail_parameters:
                print('tcc was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_tcc(args, data)

        if 'lmhc' in parameters:
            if 'lcc' not in avail_parameters or 'mcc' not in avail_parameters \
                or 'hcc' not in avail_parameters:
                print('lmhc was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_lmhc(args, data)

        if 'snow' in parameters:
            if 'snow' not in avail_parameters:
                print('snow was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_snow(args, data)

        if 'ws' in parameters:
            if 'ws' not in avail_parameters:
                print('ws was not found in available parameters: {},\
                       cannot plot'.format(avail_parameters), flush=True)
                sys.exit(1)
            self.plot_ws(args, data)

        return


    def projections(self,args:argparse.Namespace) -> None:
        """Set projection for plotting

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        """
        if args.area == 'dk':
            self.projection = ccrs.AlbersEqualArea(central_longitude=11.0, central_latitude=0.0,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [6, 16, 53, 59]

        if args.area == 'gl':
            self.projection = ccrs.TransverseMercator(central_longitude=-50.0,
                                    central_latitude=75.0, false_easting=0.0,
                                    false_northing=0.0,)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-75, -15, 58, 88]


        if args.area == 'neu':
            self.projection = ccrs.AlbersEqualArea(central_longitude=11.0, central_latitude=0.0,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-2.1, 20, 51, 65]

        if args.area == 'disko':
            self.projection = ccrs.AlbersEqualArea(central_longitude=-51.7, central_latitude=70,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-56, -46.5, 66.33, 71.52]

        if args.area == 'sjalland':
            self.projection = ccrs.AlbersEqualArea(central_longitude=11.0, central_latitude=0.0,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [10, 13.4, 54.3, 56.45]

        if args.area == 'europe':
            self.projection = ccrs.AlbersEqualArea(central_longitude=11.0, central_latitude=0.0,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-20, 30, 40, 75]

        if args.area == 'faroes':
            self.projection = ccrs.AlbersEqualArea(central_longitude=353.0, central_latitude=62.0,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-8, -6, 61.3, 62.5]

        if args.area == 'scoresbysund':
            self.projection = ccrs.AlbersEqualArea(central_longitude=-25., central_latitude=71.5,
                                    false_easting=0.0, false_northing=0.0,
                                    standard_parallels=(20.0, 50.0), globe=None)
            self.data_crs = ccrs.PlateCarree()
            self.extent = [-30.5, -20.5, 69.7, 73.7]

        return


    def plot_t2m(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot 2m temperature

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        # Fix that pcolormesh uses cell lower left corners
        clons, clats = data['lon'], data['lat']
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.t2m.colors)
        levels = [k for k in levels_and_colors.t2m.levels]
        contour_levels = [k for k in levels_and_colors.t2m.contour_levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['t2m'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'2m Temperature')

            cs = plt.pcolormesh(plons, plats, data['t2m'][k,:,:],
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$^\circ C$", rotation=270)

            cl = plt.contour(clons,clats,data['t2m'][k,:,:].values,
                             colors='black',
                             levels=contour_levels,
                             linewidths=0.7,
                             transform=self.data_crs)

            fig.canvas.draw()

            figure_name = "{}/T2M_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            [cl.collections[k].remove() for k in range(len(cl.collections))]
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_td2m(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot 2m dew point temperature

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """
        # Fix that pcolormesh uses cell lower left corners
        clons, clats = data['lon'], data['lat']
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.t2m.colors)
        levels = [k for k in levels_and_colors.t2m.levels]
        contour_levels = [k for k in levels_and_colors.t2m.contour_levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['td2m'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'2m Dew Point Temperature')

            cs = plt.pcolormesh(plons, plats, data['td2m'][k,:,:],
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$^\circ C$", rotation=270)

            cl = plt.contour(clons,clats,data['td2m'][k,:,:].values,
                             colors='black',
                             levels=contour_levels,
                             linewidths=0.7,
                             transform=self.data_crs)

            fig.canvas.draw()

            figure_name = "{}/TD2M_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            [cl.collections[k].remove() for k in range(len(cl.collections))]
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_w10m(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot 10m wind speed

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """
        # Fix that pcolormesh uses cell lower left corners
        clons, clats = np.array(data['lon']), np.array(data['lat'])
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.w10m.colors)
        levels = [k for k in levels_and_colors.w10m.levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        barbs_opts = {'height': 0.7, 'spacing': 0.2}
        bt = self.barbs_thin(clons)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['u10m'][k,:,:]): continue
            if self.check_for_empty_array(data['v10m'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'10m Wind')

            u = data['u10m'][k,:,:].values
            v = data['v10m'][k,:,:].values

            w10m = np.sqrt(u*u+v*v)

            cs = plt.pcolormesh(plons, plats, w10m,
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$m/s$", rotation=270)


            axes.barbs(clons[::bt,::bt], clats[::bt,::bt],
                       u[::bt,::bt], v[::bt,::bt],
                       length=5,
                       sizes=barbs_opts,
                       linewidth=0.95,
                       transform=self.data_crs,
                       color='#404040')

            fig.canvas.draw()

            figure_name = "{}/W10M_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_precip(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot precipitation

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """
        # Fix that pcolormesh uses cell lower left corners
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.precip.colors)
        levels = [k for k in levels_and_colors.precip.levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['precip'][k,:,:]): continue
            precip = data['precip'][k,:,:].values

            self.add_title(axes,valid_time,analysis,'Precipitation')

            cs = plt.pcolormesh(plons, plats, precip,
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$mm$", rotation=270)

            fig.canvas.draw()

            figure_name = "{}/PRECIP_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_slp(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot sea level pressure

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """
        # Fix that pcolormesh uses cell lower left corners
        clons, clats = data['lon'], data['lat']

        levels = [k for k in levels_and_colors.slp.levels]

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['slp'][k,:,:]): continue
            slp = data['slp'][k,:,:].values

            self.add_title(axes,valid_time,analysis,'Sea Level Pressure')

            cl = plt.contour(clons,clats,slp,
                             colors='black',
                             levels=levels,
                             linewidths=0.8,
                             transform=self.data_crs)

            fig.canvas.draw()

            figure_name = "{}/SLP_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            [cl.collections[k].remove() for k in range(len(cl.collections))]
            print("-- {}".format(figure_name), flush=True)

        return

    def plot_tcc(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot total cloud cover

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        # Fix that pcolormesh uses cell lower left corners
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.tcc.colors)
        levels = [k for k in levels_and_colors.tcc.levels]
        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['tcc'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'Total Cloud Cover')

            cs = plt.pcolormesh(plons, plats, data['tcc'][k,:,:],
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$\%$", rotation=270)

            fig.canvas.draw()

            figure_name = "{}/TCC_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_lmhc(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot low, medium and high cloud cover

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        # Fix that pcolormesh uses cell lower left corners
        plons, plats = self.get_pcolormesh_center_coordinates(data)

        lcc_colors = ListedColormap(levels_and_colors.lcc.colors)
        lcc_levels = [k for k in levels_and_colors.lcc.levels]
        lcc_norm = self.color_norm(lcc_levels)

        mcc_colors = ListedColormap(levels_and_colors.mcc.colors)
        mcc_levels = [k for k in levels_and_colors.mcc.levels]
        mcc_norm = self.color_norm(mcc_levels)

        hcc_colors = ListedColormap(levels_and_colors.hcc.colors)
        hcc_levels = [k for k in levels_and_colors.hcc.levels]
        hcc_norm = self.color_norm(hcc_levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['lcc'][k,:,:]): continue
            if self.check_for_empty_array(data['mcc'][k,:,:]): continue
            if self.check_for_empty_array(data['hcc'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'Low Medium High clouds')


            #TODO: pcolormesh with multiple layers
            hcc = data['hcc'][k,:,:].values
            hcc[np.where(hcc<hcc_levels[0])] = np.nan
            cs_hcc = plt.pcolormesh(plons, plats, hcc,
                                cmap=hcc_colors,
                                norm=hcc_norm,
                                transform=self.data_crs)

            mcc = data['mcc'][k,:,:].values
            mcc[np.where(mcc<mcc_levels[0])] = np.nan
            cs_mcc = plt.pcolormesh(plons, plats, data['mcc'][k,:,:],
                                cmap=mcc_colors,
                                norm=mcc_norm,
                                transform=self.data_crs)

            lcc = data['lcc'][k,:,:].values
            lcc[np.where(lcc<lcc_levels[0])] = np.nan
            cs_lcc = plt.pcolormesh(plons, plats, lcc,
                                cmap=lcc_colors,
                                norm=lcc_norm,
                                transform=self.data_crs)

            #cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            #cb.set_label(r"$\%$", rotation=270)

            fig.canvas.draw()

            figure_name = "{}/LMHC_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            #cb.remove()

            cs_lcc.remove()
            cs_mcc.remove()
            cs_hcc.remove()

            print("-- {}".format(figure_name), flush=True)

        return


    def plot_snow(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot snow cover

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.precip.colors)
        levels = [k for k in levels_and_colors.precip.levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['snow'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'snow')

            cs = plt.pcolormesh(plons, plats, data['snow'][k,:,:],
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
#            cb = plt.colorbar(cs, fraction=0.046, pad=0.04)
            cb.set_label(r"$mm$", rotation=270)

            fig.canvas.draw()

            figure_name = "{}/SNOW_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            print("-- {}".format(figure_name), flush=True)

        return


    def plot_ws(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot wind speed

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        data : xr.Dataset
            Data to plot
        """

        plons, plats = self.get_pcolormesh_center_coordinates(data)

        colors = ListedColormap(levels_and_colors.w10m.colors)
        levels = [k for k in levels_and_colors.w10m.levels]

        norm = self.color_norm(levels)

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        fig, axes = self.fig_ax(10, 8, subplot_kw={'projection': self.projection})

        self.add_coastlines(axes)

        for k in range(self.nt):
            valid_time = data['time'][k].values
            valid_time = dt.datetime.utcfromtimestamp(valid_time.astype(int) * 1e-9)

            if self.check_for_empty_array(data['ws'][k,:,:]): continue

            self.add_title(axes,valid_time,analysis,'WindSpeed (ws)')

            ws = data['ws'][k,:,:].values

            cs = plt.pcolormesh(plons, plats, ws,
                                cmap=colors,
                                norm=norm,
                                transform=self.data_crs)

            cb = plt.colorbar(cs, fraction=0.046, pad=0.04, ticks=levels)
            cb.set_label(r"$m/s$", rotation=270)

            fig.canvas.draw()

            figure_name = "{}/WS_{}-{}.png".format(args.output_dir,
                                                    analysis.strftime('%Y%m%d_%H%M'),
                                                    valid_time.strftime('%Y%m%d_%H%M'))
            plt.savefig(figure_name)
            cb.remove()
            cs.remove()
            print("-- {}".format(figure_name), flush=True)

        return



    def fig_ax(self, w:int, h:int, **kwargs:dict) -> tuple:
        """Get figure and axes

        Parameters
        ----------
        w : int
            Width of figure
        h : int
            Height of figure

        Returns
        -------
        tuple
            Figure and axes
        """
        fig = plt.figure(figsize=(w, h))
        ax = fig.subplots(1,1, **kwargs)
        ax.add_feature(cfeature.BORDERS, linestyle=':')
        return fig, ax


    def add_contour(self, ax, X:any, Y:any, data:any, levels:list, **kwargs) -> None:
        """Add contour to plot

        Parameters
        ----------
        ax : _type_
            Axes to add contour to
        X : any
            X coordinates
        Y : any
            y coordinates
        data : any
            Data to plot
        levels : list
            List of levels

        Returns
        -------
        _type_
            contourset
        """
        cc = plt.contour(X, Y, data, levels, **kwargs)
        ax.clabel(cc, fmt='%2.0f', inline=True, fontsize=10)
        return cc


    def add_coastlines(self, ax:plt.subplots, **kwargs:dict) -> tuple:
        """Add coastlines to plot

        Parameters
        ----------
        ax : plt.subplots
            Axes to add coastlines to

        Returns
        -------
        tuple
            extent and coastlines
        """
        # set_extent causes segmentation faults on some old cartopy installations
        extent = ax.set_extent(self.extent, self.data_crs)
        coastline = ax.coastlines(resolution='10m', color=(0.2,0.2,0.2), linewidth=0.7)
        return extent, coastline


    def add_title(self, ax:plt.subplots, validtime:dt.datetime,
                  analysis:dt.datetime, headline:str, **kwargs:dict) -> tuple:
        """Add title to plot

        Parameters
        ----------
        ax : plt.subplots
            axes to add title to
        validtime : dt.datetime
            Valid Time of data
        analysis : dt.datetime
            Analysis Time of data
        headline : str
            Headline to add to title

        Returns
        -------
        tuple
            Title_left, Title_center, Title_right
        """
        title_left = ax.set_title(validtime.strftime('Valid: %Y-%m-%d %H:%Mz'),
                                  fontsize=10, loc='left')
        title_center = ax.set_title(headline, fontsize=9, loc='center')
        title_right = ax.set_title(analysis.strftime('Analysis: %Y-%m-%d %H:%Mz'),
                                   fontsize=10, loc='right')
        return title_left, title_center, title_right


    def get_pcolormesh_center_coordinates(self, data:xr.Dataset) -> tuple:
        """Get pcolormesh center coordinates

        Parameters
        ----------
        data : xr.Dataset
            Data holding lon and lat

        Returns
        -------
        tuple
            lons, lats of center of each cell
        """
        # Subtract 1/2 the grid size from both lon and lat arrays
        dlon = (data['lon'][0,1] - data['lon'][0,0]).values
        dlat = (data['lat'][1,0] - data['lat'][0,0]).values
        lons = data['lon'].values - dlon/2
        lats = data['lat'].values - dlat/2
        # Add 1 grid spacing to the right column of lon array and concatenate it
        # as an additional column to the right
        lons = np.c_[ lons, lons[:,-1]+dlon ]
        # Duplicate the bottom row of the lon array and concatenate it to the bottom
        lons = np.r_[ lons, [lons[-1,:]] ]
        # Duplicate the right-most column of lats array and concatenate it on the right
        lats = np.c_[ lats, lats[:,-1] ]
        # Add 1 grid spacing to the bottom row of lat array and concatenate it
        # as an additional row below
        lats = np.r_[ lats, [lats[-1,:]+dlat] ]
        return lons, lats


    def color_norm(self, levels:list) -> matplotlib.colors.BoundaryNorm:
        """Get the color norm

        Parameters
        ----------
        levels : list
            Levels to use for color norm

        Returns
        -------
        matplotlib.colors.BoundaryNorm
            color norm
        """
        len_lab = len(levels)
        norm_bins = np.sort([*levels])
        norm = matplotlib.colors.BoundaryNorm(norm_bins, len_lab, clip=True)
        return norm


    def check_for_empty_array(self, da:xr.DataArray) -> bool:
        """Check if array is empty

        Parameters
        ----------
        da : xr.DataArray
            Array of data

        Returns
        -------
        bool
            If empty return True, False otherwise
        """
        empty = False
        flat = da.values.ravel()
        notna = flat[~np.isnan(flat)]
        if len(notna) == 0:
            empty = True
        return empty


    def barbs_thin(self, clons:np.array) -> int:
        """Thin barbs

        Parameters
        ----------
        clons : np.array
            longitudes for barbs

        Returns
        -------
        int
            Thinning factor to use for barbs
        """
        dx = abs(clons.flatten()[0] - clons.flatten()[1])
        thinner = int(0.5/dx)
        return thinner


class levels_and_colors:
    """Class for levels and colors
    """

    class t2m:
        """Class for t2m levels and colors
        """
        levels=[-24,-22,-20,-18,-16,-14,-12,-10,-8,-6,-4,-2,0,2,4,6,8,10,12,14,16,18,
                22,24,26,28,30,32,34,36,38,40,42]

        colors = [(0.14, 0.00, 0.15),(0.31, 0.00, 0.33), (0.49, 0.00, 0.54), (0.71, 0.00, 0.77),
                      (0.93, 0.00, 1.00),(0.70, 0.00, 1.00),(0.49, 0.00, 1.00),(0.30, 0.00, 1.00),
                      (0.13, 0.00, 1.00),(0.00, 0.00, 1.00),(0.05, 0.25, 1.00),(0.11, 0.49, 1.00),
                      (0.16, 0.73, 1.00),(0.22, 1.00, 1.00),(0.24, 1.00, 0.71),(0.24, 1.00, 0.45),
                      (0.24, 0.94, 0.19),(0.16, 0.85, 0.00),(0.08, 0.78, 0.00),(1.00, 1.00, 0.00),
                      (0.98, 0.88, 0.00),(0.98, 0.76, 0.00),(0.97, 0.64, 0.00),(0.97, 0.53, 0.00),
                      (0.96, 0.42, 0.00),(0.96, 0.31, 0.00),(0.96, 0.22, 0.00),(0.96, 0.12, 0.00),
                      (0.96, 0.00, 0.00),(0.95, 0.00, 0.00),(0.95, 0.00, 0.13),(0.93, 0.00, 0.33),
                      (0.96, 0.00, 0.54),(0.96, 0.00, 0.77)]

        contour_levels = [-10, -5, 0, 5, 10, 15, 20, 25]

    class w10m:
        """Class for w10m levels and colors
        """
        levels=[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,
                27,28,29,30,31,32,33,34,35]

        colors =  [(0.00,0.00,1.00),(0.00,0.00,1.00),(0.04,0.25,1.00),(0.08,0.37,1.00),
                       (0.10,0.49,1.00),(0.14,0.60,1.00),(0.16,0.73,1.00),(0.20,0.88,0.93),
                       (0.21,0.90,0.53),(0.21,0.91,0.33),(0.21,0.91,0.10),(0.22,0.96,0.00),
                       (0.24,1.00,0.00),(1.00,1.00,0.00),(0.99,0.87,0.00),(0.98,0.76,0.00),
                       (0.97,0.64,0.00),(0.97,0.53,0.00),(0.85,0.42,0.00),(0.85,0.32,0.00),
                       (0.80,0.22,0.03),(0.70,0.13,0.06),(0.62,0.05,0.07),(0.58,0.00,0.08),
                       (0.62,0.00,0.15),(0.71,0.00,0.33),(0.96,0.00,0.54),(0.96,0.00,0.77),
                       (0.96,0.00,1.00),(0.82,0.00,0.90),(0.70,0.00,0.77),(0.60,0.00,0.65),
                       (0.49,0.00,0.54),(0.39,0.00,0.44),(0.31,0.00,0.33)]

    class precip:
        """Class for precip levels and colors
        """
        # 17 levels
        levels = [0.0, 0.1, 0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 5.0, 7.0, 10.0, 15.0,
                  20.0, 25.0, 30.0, 40.0, 50.0]

        # 16 colors
        colors = [(1.00, 1.00, 1.00), (0.84, 0.69, 0.63), (0.72, 0.56, 0.51), (0.80, 0.86, 1.00),
                  (0.65, 0.73, 1.00), (0.49, 0.63, 1.00), (0.25, 0.52, 0.97), (0.00, 0.00, 0.98),
                  (0.13, 0.57, 0.09), (0.20, 0.71, 0.18), (0.41, 0.80, 0.36), (0.68, 0.99, 0.36),
                  (1.00, 1.00, 0.08), (0.99, 0.57, 0.05), (0.86, 0.00, 0.02), (0.68, 0.00, 0.02)]


    class slp:
        """Class for slp levels
        """
        # 17 levels
        levels = np.arange(950,1050,2)


    class tcc:
        """Class for tcc levels and colors
        """
        # 9 levels
        levels=[0.0, 1/8, 2/8, 3/8, 4/8, 5/8, 6/8, 7/8, 8/8]

        # 8 Colors
        colors = [(1.00, 1.00, 1.00), (0.95, 0.95, 0.95), (0.90, 0.90, 0.90), (0.80, 0.80, 0.80),
                  (0.70, 0.70, 0.70), (0.50, 0.50, 0.50), (0.40, 0.40, 0.40), (0.30, 0.30, 0.30)]

    class lcc:
        """Class for lcc levels and colors
        """
        # 9 levels
        levels=[2/8, 3/8, 4/8, 5/8, 6/8, 7/8, 8/8]

        # 8 Colors
        colors = [(0.70, 0.70, 0.70), (0.60, 0.60, 0.60),
                  (0.50, 0.50, 0.50), (0.40, 0.40, 0.40), (0.30, 0.30, 0.30), (0.20, 0.20, 0.20)]

    class mcc:
        """Class for mcc levels and colors
        """
        # 9 levels
        levels=[2/8, 3/8, 4/8, 5/8, 6/8, 7/8, 8/8]

        # 8 Colors
        colors = [(0.00, 0.90, 0.00), (0.00, 0.80, 0.00),
                  (0.00, 0.70, 0.00), (0.00, 0.60, 0.00), (0.00, 0.50, 0.00), (0.00, 0.40, 0.00)]

    class hcc:
        """Class for hcc levels and colors
        """
        # 9 levels
        levels=[2/8, 3/8, 4/8, 5/8, 6/8, 7/8, 8/8]

        # 8 Colors
        colors = [(0.00, 0.00, 0.90), (0.00, 0.00, 0.80),
                  (0.00, 0.00, 0.70), (0.00, 0.00, 0.60), (0.00, 0.00, 0.50), (0.00, 0.00, 0.40)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.arguments
Called from dlotter.__main__
"""
import sys
import argparse
from argparse import ArgumentDefaultsHelpFormatter


class MyParser(argparse.ArgumentParser):
    """Parser for dlotter arguments

    Parameters
    ----------
    argparse : argparse.ArgumentParser
        Parser for dlotter arguments
    """
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


class arguments:
    """Holds all the arguments for dlotter
    """

    def __init__(self) -> None:
        """Constructor for arguments class
        """

        return


    def get_args(self, sysargs):
        """Get arguments from command line

        Parameters
        ----------
        sysargs : sys.argv
            Arguments given to dlotter on command line

        Returns
        -------
        argparse.Namespace
            Namespace with all the arguments
        """

        parent_parser = MyParser(
            description='Plot data quick and dirty from NWP output',
            formatter_class=ArgumentDefaultsHelpFormatter)

        subparser = parent_parser.add_subparsers(dest="cmd")

        # Parser for NWP Deterministic
        parser_plot = subparser.add_parser('plot', help='Plot NWP output')

        parser_plot.add_argument('-p',
                        '--parameters',
                        metavar='PARAMETERS',
                        type=str,
                        help='Parameters to plot. Seperate with ":",\
                            eg: "t2m:w10m:precip:slp:td2m:tcc:lmhc".',
                        required=True)

        parser_plot.add_argument('-f',
                        '--filetype',
                        metavar='FILETYPE',
                        type=str,
                        help='What filetype are we using? (Options are: grib2, nc)',
                        default='grib2',
                        required=False)

        parser_plot.add_argument('-d',
                        '--directory',
                        type=str,
                        help='directory to read data from',
                        default='.')

        parser_plot.add_argument('--prefix',
                        type=str,
                        help='Set to prefix of files if any',
                        default='',
                        required=False)

        parser_plot.add_argument('--postfix',
                        type=str,
                        help='Set to postfix of files if any',
                        default='',
                        required=False)

        parser_plot.add_argument('-o',
                        '--output-dir',
                        metavar='OUTDIR',
                        type=str,
                        help='Directory to place output into',
                        default='.',
                        required=False)

        parser_plot.add_argument('-l',
                        '--limit-files',
                        metavar='LIMIT',
                        type=int,
                        help='Only use the first LIMIT files. If set to 0, not limit is used. \
                            If Limit > 0, files will be sorted by name first',
                        default=0,
                        required=False)

        parser_plot.add_argument('-a',
                        '--area',
                        metavar='AREA',
                        type=str,
                        help='Over which area to plot (Options are: dk, neu, sjalland, disko)',
                        default="dk",
                        required=False)

        parser_plot.add_argument('--verbose',
                        action='store_true',
                        help='Verbose output',
                        default=False)

        # Parser for EPS Meteogram
        parser_epsmeteogram = subparser.add_parser('epsmeteogram', help='Plot EPS meteogram')

        parser_epsmeteogram.add_argument('-d',
                            '--directory',
                            type=str,
                            help='directory to read data from',
                            default='.')

        parser_epsmeteogram.add_argument('-m',
                            '--members',
                            type=int,
                            help='Number of members',
                            required=True)

        parser_epsmeteogram.add_argument('-f',
                            '--files-per-member',
                            type=int,
                            help='Number of gribfiles per member',
                            required=True)

        parser_epsmeteogram.add_argument('--latlon',
                            type=str,
                            help='Coordinates to plot. Seperate with ":", eg: "55,12:60,14"',
                            required=True)

        parser_epsmeteogram.add_argument('--prefix',
                            type=str,
                            help='Set to prefix of files if any',
                            default='',
                            required=False)

        parser_epsmeteogram.add_argument('--postfix',
                            type=str,
                            help='Set to postfix of files if any',
                            default='',
                            required=False)

        parser_epsmeteogram.add_argument('-o',
                            '--output-dir',
                            metavar='OUTDIR',
                            type=str,
                            help='Directory to place output into',
                            default='.',
                            required=False)

        parser_epsmeteogram.add_argument('--verbose',
                            action='store_true',
                            help='Verbose output',
                            default=False)


        if len(sysargs)==1:
            parent_parser.print_help()
            sys.exit(2)

        args = parent_parser.parse_args()

        return args


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.read
Called from dlotter.__main__
"""
import argparse
import xarray as xr
import eccodes as ec
import netCDF4 as nc
import pygrib
import numpy as np
import datetime as dt
from dmit import regrot
from typing import Union

class grib2Read:
    """Class for reading grib files
    """

    def __init__(self, args:argparse.Namespace, files_to_read:list) -> None:
        """Constructor for grib2Read class

        Parameters
        ----------
        args : argparse.Namespace
            Namespace containing all the arguments

        files_to_read : list
            List of files to read

        """
        if args.verbose:
            print("Reading GRIB2", flush=True)

        self.parameters = args.parameters

        self.set_bools()

        self.data = self.read(args, files_to_read)

        return


    def set_bools(self) -> None:
        """Set bools for use in the module
        """

        self.search_t2m = False
        self.found_t2m = False
        if 't2m' in self.parameters:
            self.search_t2m = True

        self.search_td2m = False
        self.found_td2m = False
        if 'td2m' in self.parameters:
            self.search_td2m = True

        self.search_uv=False
        self.found_u = False
        self.found_v = False
        self.found_uv=False
        if 'w10m' in self.parameters:
            self.search_uv = True

        self.search_precip = False
        self.found_precip = False
        if 'precip' in self.parameters:
            self.search_precip = True

        self.search_slp = False
        self.found_slp  = False
        if 'slp' in self.parameters:
            self.search_slp = True

        self.search_tcc = False
        self.found_tcc  = False
        if 'tcc' in self.parameters:
            self.search_tcc = True

        self.search_lmhc = False
        self.found_lcc = False
        self.found_mcc = False
        self.found_hcc = False
        if 'lmhc' in self.parameters:
            self.search_lmhc = True

        self.search_snow = False
        self.found_snow = False
        if 'snow' in self.parameters:
            self.search_snow = True

        self.search_ws = False
        self.found_ws = False
        if 'ws' in self.parameters:
            self.search_ws = True

        return


    def read(self, args:argparse.Namespace, files_to_read:list) -> xr.Dataset:
        """Fetch data from the gribfile and return an xarray.Dataset()

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter.__main__
        files_to_read : list
            List of str with paths to gribfiles

        Returns
        -------
        xr.Dataset
            Dataset with found parameters
        """

        grid_idx = 0 # Set to 0
        lats, lons = self.get_latlons(files_to_read[grid_idx])

        Nt = len(files_to_read)
        Nt_coords = np.zeros(Nt, dtype=dt.datetime)

        if self.search_t2m: t2m = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
        if self.search_td2m: td2m = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        if self.search_uv:
            u10 = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
            v10 = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        if self.search_precip: precip = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
        if self.search_slp: slp = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
        if self.search_tcc: tcc = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        if self.search_lmhc:
            lcc = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
            mcc = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
            hcc = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        if self.search_snow: snow = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        if self.search_ws: ws = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)


        for k,f in enumerate(files_to_read):
            gids = self.get_gids(f)

            time_gid = gids[0]

            ec.codes_set(time_gid, 'stepUnits', 'm')

            date = ec.codes_get(time_gid, 'dataDate')
            time = ec.codes_get(time_gid, 'dataTime')
            lead = ec.codes_get(time_gid, 'step')

            analysis = dt.datetime.strptime("{:d}-{:04d}".format(date,time), '%Y%m%d-%H%M')
            forecast = analysis + dt.timedelta(minutes=lead)
            Nt_coords[k] = forecast

            for _, gid in enumerate(gids):
                shortName = ec.codes_get(gid, 'shortName')
                level = ec.codes_get(gid, 'level')
                typeOfLevel = ec.codes_get(gid, 'typeOfLevel')
                levelType = ec.codes_get(gid, 'levelType')

                Ni = ec.codes_get(gid, 'Ni')
                Nj = ec.codes_get(gid, 'Nj')

                if levelType=='103': levelType='sfc' # For grib2, leveltype 103 is surface

                if self.search_t2m and (shortName=='t' or shortName=='2t') and level==2 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_t2m = True
                    t2m[k,:,:] = values.reshape(Nj, Ni)

                if self.search_td2m and (shortName=='td' or shortName=='2td') and level==2 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_td2m = True
                    td2m[k,:,:] = values.reshape(Nj, Ni)

                if self.search_uv and (shortName=='u' or shortName=='10u') and level==10 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_u = True
                    u10[k,:,:] = values.reshape(Nj, Ni)

                if self.search_uv and (shortName=='v' or shortName=='10v') and level==10 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_v = True
                    v10[k,:,:] = values.reshape(Nj, Ni)

                if self.search_precip and (shortName=='tp' or shortName=='tprate') \
                                        and level==0 and typeOfLevel=='heightAboveGround' \
                                        and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_precip = True
                    precip[k,:,:] = values.reshape(Nj, Ni)

                if self.search_slp and (shortName=='pres') and level==0 and \
                                        typeOfLevel=='heightAboveSea' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_slp = True
                    slp[k,:,:] = values.reshape(Nj, Ni)

                if self.search_lmhc and (shortName=='lcc') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_lcc = True
                    lcc[k,:,:] = values.reshape(Nj, Ni)

                if self.search_lmhc and (shortName=='mcc') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_mcc = True
                    mcc[k,:,:] = values.reshape(Nj, Ni)

                if self.search_lmhc and (shortName=='hcc') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_hcc = True
                    hcc[k,:,:] = values.reshape(Nj, Ni)

                if self.search_tcc and (shortName=='tcc') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_tcc = True
                    tcc[k,:,:] = values.reshape(Nj, Ni)

                if self.search_snow and (shortName=='tpsolid') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_snow = True
                    snow[k,:,:] = values.reshape(Nj, Ni)

                if self.search_ws and (shortName=='ws') and level==10 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='sfc':
                    values = ec.codes_get_values(gid)
                    self.found_ws = True
                    ws[k,:,:] = values.reshape(Nj, Ni)

                ec.codes_release(gid)

        ds_grib = xr.Dataset(coords={"lat": (["x","y"], lats),
                                     "lon": (["x","y"], lons),
                                     "time": (["t"], Nt_coords)})

        if self.found_t2m: ds_grib['t2m'] = (['time', 'lat', 'lon'], t2m - 273.15 )
        if self.found_td2m: ds_grib['td2m'] = (['time', 'lat', 'lon'], td2m - 273.15 )
        if self.found_u: ds_grib['u10m'] = (['time', 'lat', 'lon'], u10 )
        if self.found_u: ds_grib['v10m'] = (['time', 'lat', 'lon'], v10 )
        if self.found_precip: ds_grib['precip'] = (['time', 'lat', 'lon'], precip )
        if self.found_slp: ds_grib['slp'] = (['time', 'lat', 'lon'], slp * 0.01)
        if self.found_tcc: ds_grib['tcc'] = (['time', 'lat', 'lon'], tcc )
        if self.found_lcc: ds_grib['lcc'] = (['time', 'lat', 'lon'], lcc )
        if self.found_mcc: ds_grib['mcc'] = (['time', 'lat', 'lon'], mcc )
        if self.found_hcc: ds_grib['hcc'] = (['time', 'lat', 'lon'], hcc )
        if self.found_snow: ds_grib['snow'] = (['time', 'lat', 'lon'], snow )
        if self.found_ws: ds_grib['ws'] = (['time', 'lat', 'lon'], ws )

        if len(list(ds_grib.data_vars)) == 0:
            raise SystemExit('No variables found. This can be due to missing tables \
                              in ECCODES_DEFINITION_PATH or that the requested keys \
                              are not yet implemented')

        ds_grib = self.sort_by_time(ds_grib)

        return ds_grib


    def sort_by_time(self, dataarray:xr.Dataset) -> xr.Dataset:
        """Sort dataarray by time.

        Parameters
        ----------
        dataarray : xr.Dataset
            Dataarray to sort.

        Returns
        -------
        xr.Dataset
            Sortet dataarray.
        """

        nt = dataarray.dims['time']
        parameters = list(dataarray.data_vars)

        time = dataarray['time'].values
        idx = np.argsort(time)

        da = dataarray.sortby('time')

        for p in parameters:
            for k in range(nt):
                da[p][k,:,:] = dataarray[p][idx[k],:,:]

        return da


    def get_latlons(self, gribfile:str) -> tuple:
        """Get latitudes and longitudes from file. Uses pygrib as
        eccodes have no easy interface for that.

        Parameters
        ----------
        gribfile : str
            Path to gribfile

        Returns
        -------
        tuple
            tuple of latitudes, longitudes
        """

        gr = pygrib.open(gribfile)
        g = gr[1]

        if g['gridType'] == 'rotated_ll':
            print('Found rotated grid, discarding pygrib extraction')
            latdim = g.Nj
            londim = g.Ni

            latFirst = g.latitudeOfFirstGridPointInDegrees
            latLast  = g.latitudeOfLastGridPointInDegrees
            lonFirst = (g.longitudeOfFirstGridPointInDegrees % 180.)-180.
            lonLast  = g.longitudeOfLastGridPointInDegrees
            # dy = g.jDirectionIncrementInDegrees
            # dx = g.iDirectionIncrementInDegrees
            latPole = g.latitudeOfSouthernPoleInDegrees
            lonPole = g.longitudeOfSouthernPoleInDegrees

            lons, lats = np.meshgrid(np.linspace(
                lonFirst, lonLast, londim), np.linspace(latFirst, latLast, latdim))

            lons, lats = regrot.rot_to_reg(lonPole, latPole, lons, lats)
        else:
            lats, lons = gr[1].latlons()

        gr.close()

        return lats, lons


    def get_gids(self, gribfile:str, TextIOWrapper:bool=False) -> list:
        """Get GribIDs (gid) for all the messages in one gribfile

        Parameters
        ----------
        gribfile : str
            path to gribfile
        TextIOWrapper : bool
            if file is open send the object instead and set TextIOWrapper to True

        Returns
        -------
        list
            list of grib-ids
        """

        if not TextIOWrapper:
            f = open(gribfile, 'rb')
            msg_count = ec.codes_count_in_file(f)
            gids = [ec.codes_grib_new_from_file(f) for i in range(msg_count)]
            f.close()
        else:
            #gribfile has already been opened into a TextIOWrapper in this case
            msg_count = ec.codes_count_in_file(gribfile)
            gids = np.zeros(msg_count, dtype=int)
            for i in range(msg_count):
                gids[i] = ec.codes_grib_new_from_file(gribfile)

        return gids



class netcdf2read:
    """Class for reading netcdf files.
    """

    def __init__(self, args:argparse.Namespace, files_to_read:list) -> None:
        """Constructor for netcdf2read class.

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from command line
        files_to_read : list
            List of files to read
        """
        if args.verbose:
            print("Reading NetCDF", flush=True)

        self.parameters = args.parameters

        self.set_bools()

        self.data = self.read(args, files_to_read)

        return


    def set_bools(self) -> None:
        """Set booleans for which variables to read.
        """

        self.search_t2m = False
        self.found_t2m = False
        if 't2m' in self.parameters:
            self.search_t2m = True

        self.search_precip = False
        self.found_precip = False
        if 'precip' in self.parameters:
            self.search_precip = True

        return


    def sort_by_time(self, dataarray:xr.Dataset) -> xr.Dataset:
        """Sort xarray dataset by time.

        Parameters
        ----------
        dataarray : xr.Dataset
            Dataset to sort

        Returns
        -------
        xr.Dataset
            Sorted dataset
        """

        nt = dataarray.dims['time']
        parameters = list(dataarray.data_vars)

        time = dataarray['time'].values
        idx = np.argsort(time)

        da = dataarray.sortby('time')

        for p in parameters:
            for k in range(nt):
                da[p][k,:,:] = dataarray[p][idx[k],:,:]

        return da


    def read(self, args:argparse.Namespace, files_to_read:list) -> xr.Dataset:
        """Fetch data from the netcdf files and return an xarray.Dataset()

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter.__main__
        files_to_read : list
            List of str with paths to netcdf files

        Returns
        -------
        xr.Dataset
            Dataset with found parameters
        """

        lats, lons = self.get_latlons(files_to_read[0])

        Nt = len(files_to_read)
        Nt_coords = np.zeros(Nt, dtype=dt.datetime)

        if self.search_t2m: t2m = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
        if self.search_precip: precip = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)

        for k,f in enumerate(files_to_read):
            print(k,f)

            f = nc.Dataset(files_to_read[k])

            forecast = f.getncattr('ValidDate')
            forecast = dt.datetime.strptime(forecast, '%Y-%b-%d %H:%M:%S')

            Nt_coords[k] = forecast

            if self.search_t2m:
                t2m_key = self.find_relevant_key(f, 't2m')
                if t2m_key is not None:
                    t2m[k,:,:] = f[t2m_key][:,:]
                    self.found_t2m = True

            if self.search_precip:
                precip_key = self.find_relevant_key(f, 'precip')
                if precip_key is not None:
                    precip[k,:,:] = f[precip_key][:,:]
                    self.found_precip = True


            f.close()


        ds_grib = xr.Dataset(coords={"lat": (["x","y"], lats),
                                     "lon": (["x","y"], lons),
                                     "time": (["t"], Nt_coords)})

        if self.found_t2m: ds_grib['t2m'] = (['time', 'lat', 'lon'], t2m - 273.15 )
        if self.found_precip: ds_grib['precip'] = (['time', 'lat', 'lon'], precip)

        if len(list(ds_grib.data_vars)) == 0:
            raise SystemExit('No variables found. This can be due to missing tables in \
                              ECCODES_DEFINITION_PATH or that the requested keys are \
                              not yet implemented')

        ds_grib = self.sort_by_time(ds_grib)

        return ds_grib


    def get_latlons(self, netcdf_file:str) -> tuple:
        """Get latitudes and longitudes from file. Uses netcdf4-python as
        eccodes have no easy interface for that.

        Parameters
        ----------
        netcdf_file : str
            Path to netcdf file

        Returns
        -------
        tuple
            tuple of latitudes, longitudes
        """

        f = nc.Dataset(netcdf_file)

        latkey = self.find_relevant_key(f, 'latitude')
        lonkey = self.find_relevant_key(f, 'longitude')

        if latkey is None or lonkey is None:
            raise SystemExit('No lat/lon found in netcdf file')

        lats = f[latkey][:,:]
        lons = f[lonkey][:,:]

        f.close()

        return lats, lons


    def find_relevant_key(self, dataset:nc._netCDF4.Dataset, key:str) -> Union[str, None]:
        """Check if a key is in the netcdf file.

        Parameters
        ----------
        nc._netCDF4.Dataset
            Netcdf file
        key : str
            Key to search for

        Returns
        -------
        str
            None if no key is found, otherwise returns the best guess key
        """

        available_keys = dataset.variables.keys()

        return_key = None

        if key == 'latitude':
            if 'lat' in available_keys: return_key = 'lat'
            elif 'latitude' in available_keys: return_key = 'latitude'
            elif 'LAT' in available_keys: return_key = 'LAT'

        elif key == 'longitude':
            if 'lon' in available_keys: return_key = 'lon'
            elif 'longitude' in available_keys: return_key = 'longitude'
            elif 'LON' in available_keys: return_key = 'LON'

        elif key == 't2m':
            if 't2m' in available_keys: return_key = 't2m'
            elif 't2maboveground' in available_keys: return_key = 't2maboveground'
            elif 'T2M' in available_keys: return_key = 'T2M'

        elif key == 'precip':
            if 'precip' in available_keys: return_key = 'precip'
            elif 'precipitation' in available_keys: return_key = 'precipitation'
            elif 'PRECIP' in available_keys: return_key = 'PRECIP'

        return return_key



"""__init__ of dlotter
"""
import os
import sys
sys.path.insert(0, os.path.abspath('./dlotter/'))
from .prepare import prepare
from .read import grib2Read
from .plot import plot
from .arguments import arguments
from .meteogram import meteogram


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.comeps
Called from dlotter.__main__
"""
import argparse
import xarray as xr
import eccodes as ec
import numpy as np
import datetime as dt
from dmit import ostools
import dmit.calc.sun as sun
from .read import grib2Read
import gc

from scipy.spatial import cKDTree

class comeps:
    """Class for reading COMEPS data
    """

    def __init__(self, args:argparse.Namespace) -> None:
        """Constructor for comeps class

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from argparse
        """
        self.data = self.read_comeps(args)
        return


    def get_location_index(self, location:str, tree:cKDTree) -> int:
        """_summary_

        Parameters
        ----------
        location : str
            String of location with latitude and longitude
        tree : cKDTree
            scipy.spatial.cKDTree object

        Returns
        -------
        int
            Index of location in tree
        """
        location = location.split(',')

        ref_lat = float(location[0])
        ref_lon = float(location[1])

        _, idx = tree.query((ref_lat,ref_lon))

        return idx


    def read_comeps(self, args:argparse.Namespace) -> xr.Dataset:
        """Read COMEPS data

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments from argparse

        Returns
        -------
        xr.Dataset
            Dataset with COMEPS data
        """

        base_file = "{}/mbr{:03d}/{:03d}".format(args.directory,0,0)
        if not ostools.does_file_exist(base_file):
            print("Base file: {} was not found").format(base_file)

        lats, lons = grib2Read.get_latlons(self, base_file)

        no_members = args.members
        nt = args.files_per_member
        Nt_coords = np.zeros(nt, dtype=dt.datetime)

        locations = args.latlon.split(':')
        no_locations = len(locations)

        data_locations = list(zip(lats.flatten(), lons.flatten()))
        tree = cKDTree(data_locations)

        data_indexes = np.empty(no_locations, dtype=int)
        data_lats = np.empty(no_locations, dtype=np.single)
        data_lons = np.empty(no_locations, dtype=np.single)

        i=0
        for loc in locations:
            location_idx = self.get_location_index(loc, tree)
            data_indexes[i] = location_idx
            data_lats[i] = lats.flatten()[location_idx]
            data_lons[i] = lons.flatten()[location_idx]

            i+=1

        del tree

        precip = np.zeros([nt, no_members, no_locations], dtype=np.single) + np.nan
        precip_solid = np.zeros([nt, no_members, no_locations], dtype=np.single) + np.nan
        cloudcover = np.zeros([nt, no_members, no_locations], dtype=np.single) + np.nan
        visibility = np.zeros([nt, no_members, no_locations], dtype=np.single) + np.nan
        # night 0 if day, 1 if night (note not dependent on number of members)
        night = np.zeros([nt, no_locations], dtype=np.single) + np.nan

        for l in range(no_locations):
            loc_idx = data_indexes[l]

            for k in range(nt):

                for m in range(no_members):
                    if args.verbose: print("\nTime: {}/{}, Member: {}/{}".format(k,nt,m,no_members))
                    epsfile = "{}/mbr{:03d}/{:03d}".format(args.directory,m,k)
                    if not ostools.does_file_exist(epsfile): continue

                    f = open(epsfile, 'r')
                    if args.verbose: print("Reading: {}".format(epsfile))

                    gids = grib2Read.get_gids(self, f, TextIOWrapper=True)

                    time_gid = gids[0]

                    ec.codes_set(time_gid, 'stepUnits', 'm')

                    date = ec.codes_get(time_gid, 'dataDate')
                    time = ec.codes_get(time_gid, 'dataTime')
                    lead = ec.codes_get(time_gid, 'step')

                    analysis = dt.datetime.strptime(('%i-%.2i')%(date,time),'%Y%m%d-%H%M')
                    forecast = analysis + dt.timedelta(minutes=lead)

                    if k==0 and m==0:
                        # We'll save some time and only calculate one sunset and sunrise
                        sunrise, sunset = self.get_sunrise_sunset(data_lats[l], data_lons[l],
                                                                  forecast)
                    if m==0:
                        night[k,l] = self.is_it_night(sunrise, sunset, forecast)

                    Nt_coords[k] = forecast

                    for i, gid in enumerate(gids):
                        #shortName = ec.codes_get(gid, 'shortName')
                        level = ec.codes_get(gid, 'level')
                        typeOfLevel = ec.codes_get(gid, 'typeOfLevel')
                        levelType = ec.codes_get(gid, 'levelType')
                        iop = ec.codes_get(gid, 'indicatorOfParameter')

                        #Ni = ec.codes_get(gid, 'Ni')
                        #Nj = ec.codes_get(gid, 'Nj')

                        if iop==61 and level==0 and typeOfLevel=='heightAboveGround' \
                            and levelType=='sfc':

                            values = ec.codes_get_values(gid)
                            if k == 0:
                                precip[k,m,l] = values[loc_idx]
                            else:
                                precip[k,m,l] = values[loc_idx] - precip[k-1,m,l]

                        if iop==71 and level==0 and typeOfLevel=='heightAboveGround' \
                            and levelType=='sfc':

                            values = ec.codes_get_values(gid)
                            cloudcover[k,m,l] = values[loc_idx]

                        if iop==20 and level==0 and typeOfLevel=='heightAboveGround' \
                            and levelType=='sfc':

                            values = ec.codes_get_values(gid)
                            visibility[k,m,l] = values[loc_idx]

                        if iop==185 and level==0 and typeOfLevel=='heightAboveGround' \
                            and levelType=='sfc':

                            values = ec.codes_get_values(gid)
                            if k == 0:
                                precip_solid[k,m,l] = values[loc_idx]
                            else:
                                precip_solid[k,m,l] = values[loc_idx] - precip_solid[k-1,m,l]

                        ec.codes_release(gid)

                    f.close()

                # Member loop closed
                gc.collect()
            # Time loop closed
        # Location loop closed


        ds_grib = xr.Dataset(coords={"location": (["l"], np.arange(no_locations)),
                                     "member": (["m"], np.arange(no_members)),
                                     "time": (["t"], Nt_coords)})

        ds_grib['precip'] = (['time', 'member', 'locations'], precip)
        ds_grib['tcc'] = (['time', 'member', 'locations'], cloudcover)
        ds_grib['precip_solid'] = (['time', 'member', 'locations'], precip_solid)
        ds_grib['visibility'] = (['time', 'member', 'locations'], visibility)
        ds_grib['night'] = (['time','locations'], night)


        return ds_grib


    def get_sunrise_sunset(self, latitude:float, longitude:float, time:dt.datetime) -> tuple:
        """Get time of sunrise and sunset for a given location and time.

        Parameters
        ----------
        latitude : float
            Latitude of location.
        longitude : float
            Longitude of location.
        time : dt.datetime
            Time (day) of interest.

        Returns
        -------
        tuple
            Sunrise and sunset time.
        """
        sunrise = sun.calc_sun_time(longitude, latitude, time, setrise='sunrise')
        sunset = sun.calc_sun_time(longitude, latitude, time, setrise='sunset')

        return sunrise, sunset


    def is_it_night(self, sunrise:dt.datetime, sunset:dt.datetime, time:dt.datetime) -> int:
        """Get information if it is night or day.

        Parameters
        ----------
        sunrise : dt.datetime
            Time of sunrise
        sunset : dt.datetime
            Time of sunset
        time : dt.datetime
            current time

        Returns
        -------
        int
            0 if day, 1 if night
        """

        if time.hour >= sunrise.hour and time.hour < sunset.hour:
            binary_day = 0 # Day
        else:
            binary_day = 1 # Night

        return binary_day


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.meteogram
Called from dlotter.__main__
Plots meteograms
"""
import argparse
import xarray as xr
import numpy as np
import datetime as dt

import matplotlib.pyplot as plt
from matplotlib.offsetbox import OffsetImage, AnnotationBbox


class meteogram:
    """Class for plotting meteograms
    """


    def __init__(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Constructor for meteogram class

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter from command line
        data : xr.Dataset
            Data to plot
        """

        print('- Plotting', flush=True)

        self.define_icons()
        self.plot_epsmeteogram(args, data)

        print('- Done Plotting', flush=True)

        return


    def define_icons(self) -> None:
        """Define relative paths to icons
        """
        #Day
        self.sun          = 'dlotter/icons/0.png'
        self.partcloud    = 'dlotter/icons/1.png'
        self.cloudy       = 'dlotter/icons/2.png'
        self.haze         = 'dlotter/icons/3.png'

        self.rain_shower        = 'dlotter/icons/101.png'
        self.rain               = 'dlotter/icons/102.png'
        self.heavy_rain_shower  = 'dlotter/icons/111.png'
        self.heavy_rain         = 'dlotter/icons/112.png'

        self.sleet_shower       = 'dlotter/icons/201.png'
        self.sleet              = 'dlotter/icons/202.png'
        self.heavy_sleet_shower = 'dlotter/icons/211.png'
        self.heavy_sleet        = 'dlotter/icons/212.png'

        self.snow_shower        = 'dlotter/icons/301.png'
        self.snow               = 'dlotter/icons/302.png'
        self.heavy_snow_shower  = 'dlotter/icons/311.png'
        self.heavy_snow         = 'dlotter/icons/312.png'

        # Night
        self.moon               = 'dlotter/icons/1000.png'
        self.partcloud_night    = 'dlotter/icons/1001.png'
        self.cloudy_night       = 'dlotter/icons/1002.png'
        self.haze_night         = 'dlotter/icons/1003.png'

        self.rain_shower_night        = 'dlotter/icons/1010.png'
        self.rain_night               = 'dlotter/icons/1020.png'
        self.heavy_rain_shower_night  = 'dlotter/icons/1110.png'
        self.heavy_rain_night         = 'dlotter/icons/1120.png'

        self.sleet_shower_night       = 'dlotter/icons/2010.png'
        self.sleet_night              = 'dlotter/icons/2020.png'
        self.heavy_sleet_shower_night = 'dlotter/icons/2110.png'
        self.heavy_sleet_night        = 'dlotter/icons/2120.png'

        self.snow_shower_night        = 'dlotter/icons/3010.png'
        self.snow_night               = 'dlotter/icons/3020.png'
        self.heavy_snow_shower_night  = 'dlotter/icons/3110.png'
        self.heavy_snow_night         = 'dlotter/icons/3120.png'

        return


    def plot_epsmeteogram(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        """Plot EPS meteogram with weather icons

        Parameters
        ----------
        args : argparse.Namespace
            Input to dlotter from command line
        data : xr.Dataset
            Data to plot
        """
        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        x = np.arange(args.files_per_member)
        y = np.arange(args.members)
        xx, yy = np.meshgrid(x,y)

        locations = args.latlon.split(':')

        l=0
        for loc in locations:

            _, ax = plt.subplots(figsize=(14,8))
            self.add_title(ax, analysis, "location: {}".format(loc))

            k = 0
            for x0, y0 in zip(xx.flatten(), yy.flatten()):
                #filenumber = x0 #aka time-dimension
                #member = y0

                cloud = data['tcc'][x0,y0,l]
                precip = data['precip'][x0,y0,l]
                precip_solid = data['precip_solid'][x0,y0,l]
                visibility = data['visibility'][x0,y0,l]

                precip_type = self.get_precip_type(precip, precip_solid)

                night = data['night'][x0,l]

                symbol = self.get_weather_symbol(cloud, precip, precip_type, visibility, night)

                path = symbol
                k+=1

                ab = AnnotationBbox(self.getImage(path), (x0, y0), frameon=False)
                ax.add_artist(ab)

            ylabel = ['mbr{:02d}'.format(k) for k in y]
            plt.yticks(y, ylabel)

            xlabel = [(analysis+dt.timedelta(hours=int(k))).strftime('%a\n%Hz') for k in x]
            plt.xticks(x[::2], xlabel[::2])
            #plt.margins(x=0, y=10)

            ax.spines['top'].set_visible(False)
            ax.spines['right'].set_visible(False)
            ax.spines['bottom'].set_visible(False)
            ax.spines['left'].set_visible(False)
            ax.tick_params(axis='both', which='major', pad=25)

            plt.savefig('{}/epsplot_{}.png'.format(args.output_dir, l))
            l+=1

        return


    def getImage(self, path:str) -> OffsetImage:
        """Read image from file and return it as matplotlib.offsetbox.OffsetImage

        Parameters
        ----------
        path : str
            Path to image file

        Returns
        -------
        OffsetImage
            Image object
        """
        return OffsetImage(plt.imread(path), zoom=0.1)

    def fig_ax(self, w:int, h:int, **kwargs:dict) -> tuple:
        """Create figure and axes

        Parameters
        ----------
        w : int
            Width of figure
        h : int
            Height of figure

        Returns
        -------
        tuple
            Tuple of figure and axis
        """
        fig = plt.figure(figsize=(w, h))
        ax = fig.subplots(1,1, **kwargs)
        return fig, ax


    def add_title(self, ax:plt.subplots, analysis:dt.datetime, headline:str,
                  **kwargs:dict) -> tuple:
        """Add a title to the plot

        Parameters
        ----------
        ax : plt.subplots
            Subplot axes
        analysis : dt.datetime
            Analysis time of the forecast
        headline : str
            Headline to add to plot

        Returns
        -------
        tuple
            Title for center position and right position
        """
        title_center = ax.set_title(headline, fontsize=9, loc='center')
        title_right = ax.set_title(analysis.strftime('Analysis: %Y-%m-%d %H:%M'),
                                   fontsize=10, loc='right', pad=20)
        return title_center, title_right



    def get_weather_symbol(self, cloud:float, precip:float, precip_type:str,
                           visibility:float, night:float) -> str:
        """Get weather symbol

        Parameters
        ----------
        cloud : float
            Cloud fraction
        precip : float
            Amount of precipitation
        precip_type : str
            Type of precipitation
        visibility : float
            Visibility in meters
        night : float
            1 if night, 0 otherwise

        Returns
        -------
        str
            Path to icon
        """

        symbol = 0

        is_night = False
        if night==1: is_night = True

        bool_precip = False


        if precip >= 2.0: # Heavy Rain
            if precip_type == 'ra':
                symbol+=110
            elif precip_type == 'sl':
                symbol+=210
            elif precip_type == 'sn':
                symbol+=310
            bool_precip = True
        elif precip > 0.1: # Light Rain
            if precip_type == 'ra':
                symbol+=100
            elif precip_type == 'sl':
                symbol+=200
            elif precip_type == 'sn':
                symbol+=300
            bool_precip = True
        elif precip <= 0.1: # No Rain
            symbol += 0


        if visibility <= 200:
            symbol+=3
            if precip > 0.1: # If Rain + Low visibility
                symbol -= 1
        elif cloud > 0.8: # Cloudy
            symbol+=2
        elif cloud > 0.2: # Broken
            symbol+=1
        elif cloud <= 0.2 and not bool_precip: # Clear
            symbol+=0
        elif cloud <= 0.2 and bool_precip: # Clear but heavy shower
            symbol+=1

        if is_night: symbol+=1000

        icon = 'dlotter/icons/{}.png'.format(symbol)
        return icon


    def get_precip_type(self, precip:float, precip_solid:float) -> str:
        """Get type of precipitation

        Parameters
        ----------
        precip : float
            Amount of precipitation
        precip_solid : float
            Amount of solid precipitation

        Returns
        -------
        str
            Type of precipitation (ra, sn, sl)
        """

        if precip == 0:
            precip_type = 'ra' # Default value, in this case it will not be used
        elif precip != 0 and precip_solid == precip: # Everything is snow
            precip_type = 'sn'
        elif precip != 0 and precip_solid > 0 and precip > precip_solid:
            # Both rain and snow, choose sleet
            precip_type = 'sl'
        else:
            precip_type = 'ra'

        return precip_type



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter
"""

__author__ = "K. Hintz"
__copyright__ = "Danish Meteorological Institute"

__license__ = "MIT"
__version__ = "0.0.2.2"
__maintainer__ = "K. Hintz"
__email__ = "kah@dmi.dk"
__status__ = "Development"

import sys
import os
sys.path.insert(0, os.path.abspath('./dlotter/'))
import argparse

from .prepare import prepare
from .read import grib2Read, netcdf2read
from .plot import plot
from .arguments import arguments
from .meteogram import meteogram
from .comeps import comeps

from dmit import ostools

class MyParser(argparse.ArgumentParser):
    """Parser for dlotter arguments

    Parameters
    ----------
    argparse : argparse.ArgumentParser
        Parser for dlotter arguments
    """
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


if __name__ == '__main__':

    modargs = arguments()
    args = modargs.get_args(sys.argv)

    if args.verbose:
        print('---- Input Arguments ----', flush=True)
        for p in args._get_kwargs():
                print("{}: {}".format(p[0], p[1]), flush=True)
        print('---- --------------- ----', flush=True)

    # Check if output directory exist. If not try to create it.
    if not ostools.does_dir_exist(args.output_dir):
        os.makedirs(args.output_dir, exist_ok=True)


    if args.cmd == 'plot':
        prepwork = prepare(args)
        files_to_read = prepwork.files_to_read

        if args.verbose:
            print('Found the following files:', flush=True)
            print(files_to_read, flush=True)

        if args.filetype == 'grib2':
            datareader = grib2Read(args, files_to_read)
            data = datareader.data
        elif args.filetype == 'nc':
            datareader = netcdf2read(args, files_to_read)
            data = datareader.data
        else:
            print('Filetype: "{}", not supported.'.format(args.filetype), flush=True)
            sys.exit(1)

        plotwork = plot(args, data)


    if args.cmd == 'epsmeteogram':
        eps = comeps(args)
        data = eps.data

        plotwork = meteogram(args, data)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for setup dlotter
"""
import setuptools
from glob import glob

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
     name='dlotter',
     version='0.1.0',
     author="Kasper Hintz",
     author_email="kah@dmi.dk",
     description="Quick and dirty static plots of NWP output",
     long_description=long_description,
     long_description_content_type="text/markdown",
     packages=setuptools.find_packages(),
     setup_requires=[
         'wheel',
         ],
     install_requires=[
         'wheel',
         'dmit',
         'xarray',
         'numpy',
         'cartopy',
         'eccodes',
         'pygrib',
         'scipy',
         ],
     url="https://dmidk.github.io/dlotter/",
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
     include_package_data=True,
     package_data={'dlotter': glob('dlotter/icons/*.png')},
 )
